
-- Estimating the space savings with vardecimal storage format

/*
Before enabling Vardecimal storage format, you may want to know the potential reduction in the size of the table. Clearly, if the table has no decimal/numeric columns, there will be no savings. Note, that even if you have a table with decimal/numeric column types, there is no guarantee that you will be able to reduce the size of the table by enabling Vardecimal storage format. Again, this issue is similar to VARCHAR (17) vs. CHAR(17). If all the values in the column type has 17 characters, then average row length will be larger with VARCHAR(17) because it will be stored in the variable portion of the record structure. Recall, you need 2 bytes to store the offset of the variable length column. Also, if VARCAHR(17)  is the only variable length column in the table, there is another overhead of 2 bytes to store number of variable length columns in the row.  So in this case, the worst case, declaring column type as VARCAHR(17) may cost you 4 bytes more for each row than CHAR(17).  

SQL Server 2005/SP2 provides you a tool, a stored procedure, to estimate the 'reduction in row size' with Vardecimal storage format. The following example illustrates the reduction in row size for two tables that have same scheme but different data, t_decimal being the best case and t_decimal2 being the worst case (where each decimal value has max 38 digits as allowed by the declared precision) 
*/

create table t_decimal (c1 int, c2 decimal(10,2), c3 decimal (38,2), c4 varchar(10))
go

create table t_decimal2 (c1 int, c2 decimal(10,2), c3 decimal (38,2), c4 varchar(10))
go

-- insert rows into these tables. 
declare @i int
select @i = 0
while (@i < 1000) 
begin
        insert into t_decimal values (1, 0.0,0.0, 'hello')
        insert into t_decimal2 values
                (1,12345678.99,123456789012345678901234567890123499.99, 'hello')
         set @i = @i + 1
end

-- Now let us find the potential space savings for each of these tables
-- This is the best case
exec sys.sp_estimated_rowsize_reduction_for_vardecimal 't_decimal'

/*
Here is the output. Note, in this case, you can reduce the size of the row by almost 50%. Also, if you have more decimal/numeric columns, the savings will be proportionally larger.

avg_rowlen_fixed_format  avg_rowlen_vardecimal_format    row_count
--------------------------------------- --------------------------
46.00                      24.00                         1000
*/

-- this is worst case. Note in this case, the average row length actually increases
-- with Vardecimal storage format.
-- 
exec sys.sp_estimated_rowsize_reduction_for_vardecimal 't_decimal2'

/*
avg_rowlen_fixed_format  avg_rowlen_vardecimal_format   row_count
-------------------------   ---------- --------------------    ------
46.00                                 48.00                                       1000
*/

-- Enabling vardecimal storage format

/*
Enabling vardecimal storage format on a table is a two step process as follows:

First you need to enable database for Vardecimal storage format. This can be done using the stored procedure sp_db_vardecimal_storage_format. The exact command is as follows
*/

exec sp_db_vardecimal_storage_format '<dbname>', 'ON'

/*
When the above command is executed, SQL Server internally bumps the database version number but no tables are enabled for Vardecimal storage format. The database version needs to be bumped to indicate that the data in this database can potentially have a different storage format (i.e. the Vardecimal storage format). This is used to prevent attaching a Vardecimal enabled database to earlier versions of SQL Server 2005 as those versions don't know how to interpret the new storage format. You can only enable Vardecimal storage format on user databases.

To find out which database(s) is enabled for Vardecimal storage format, you can use the following command
*/

exec sp_db_vardecimal_storage_format

/*
Once you have enabled the database for Vardecimal storage format, you can now choose to enable one or more tables (based on the potential disk savings using the tool described earlier) with this new storage format as follows
*/

sp_tableoption '<table-name>', 'vardecimal storage format', 1

/*
This command, potentially an expensive one (same order as creating an index), converts all the rows in the table containing columns of type decimal/numeric to Vardecimal storage format. During this conversion, the table is locked and is not available. If the table has no clustered index, then all non-clustered indexes are rebuilt because the RIDs of the rows will change due to storage format change. However, if you have clustered index on the table, then only the non-clustered indexes containing decimal numeric column as key or included column need to be rebuilt. Note, that you cannot enable vardecimal storage format on all tables. Before enabling vardecimal storage format, SQL Server needs to make sure that we can always revert back to static storage format for decimal data and that update of decimal/numeric data always succeeds. If these conditions are not satisfied, the conversion to vardecimal storage format is denied. 

To disable Vardecimal storage format on the table, you can use the following command
*/

sp_tableoption '<table-name>', 'vardecimal storage format', 0

/*
SQL Server guarantees that you can always revert back to 'static' storage format unless you run out of disk space during conversion. Note, that the space overhead, to enable/disable Vardecimal storage format, is of the same order as building an index and it is not an online operation.

You can use the following command to find out which tables(s) has been enabled for Vardecimal storage format
*/

select objectproperty(object_id('<table-name>'), 'TableHasVarDecimalStorageFormat')

-- or
select name, object_id, type_desc
from sys.objects 
where objectproperty(object_id, N'TableHasVarDecimalStorageFormat') = 1
 


